@@ -34,7 +34,7 @@ authorization from the copyright holder.
 
 */
 
-/* $Id: synctex-luatex.h 3685 2010-05-13 07:37:47Z taco $ */
+/* $Id: synctex-luatex.h 4005 2010-11-29 11:52:43Z taco $ */
 
 #undef synchronization_field_size
 #define synchronization_field_size 1
@@ -85,7 +85,7 @@ authorization from the copyright holder.
 #define SYNCTEX_CURH static_pdf->posstruct->pos.h
 
 #define SYNCTEX_GET_JOB_NAME() makecstring(job_name)
-#define SYNCTEX_GET_LOG_NAME() (char *)xstrdup((const char*)texmf_log_name)
+#define SYNCTEX_GET_LOG_NAME() get_full_log_name()
 
 #define SYNCTEX_RULE_WD width(p)
 #define SYNCTEX_RULE_HT height(p)
@@ -107,4 +107,3 @@ authorization from the copyright holder.
 #define SYNCTEX_OUTPUT ((pdf_output_value>0)?"pdf":"dvi")
 
 #define __SyncTeX__ 1
-
